@@ -1,21 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-import sys
-import os
-import re
-import json
 import codecs
-import requests
+import json
+import os
 import pdb
+import re
+import sys
 import webbrowser
 from collections import defaultdict
+
+import requests
+
 from chgksuite.common import (
-    get_lastdir,
-    set_lastdir,
-    on_close,
-    log_wrap,
-    bring_to_front,
     get_chgksuite_dir,
+    get_lastdir,
+    get_source_dirs,
+    log_wrap,
+    set_lastdir,
 )
 
 API = "https://trello.com/1"
@@ -41,8 +42,8 @@ def upload_file(filepath, trello):
     for card in cards:
         caption = "вопрос"
         if re.search("\n! (.+?)\r?\n", card):
-            caption = re.search("\n! (.+?)\.?\r?\n", card).group(1)
-            if trello["author"] and re.search("\n@ (.+?)\.?\r?\n", card):
+            caption = re.search("\n! (.+?)\\.?\r?\n", card).group(1)
+            if trello["author"] and re.search("\n@ (.+?)\\.?\r?\n", card):
                 caption += " {}".format(re.search("\n@ (.+?)\r?\n", card).group(1))
 
         req = requests.post(
@@ -61,7 +62,7 @@ def upload_file(filepath, trello):
 
 
 def gui_trello_upload(args, sourcedir):
-    ld = get_lastdir()
+    get_lastdir()
 
     if not args.board_id:
         board_id = get_board_id()
@@ -119,16 +120,67 @@ def noanswers_line_check(line):
     )
 
 
-RE_LINK = re.compile("\\[(.+?)\\]\\((.+?)\\)")
+RE_LINK = re.compile("\\]\\(")
+
+
+def find_and_parse_link(str_, index_):
+    assert str_[index_] == "]"
+    assert str_[index_ + 1] == "("
+    mvr = index_
+    level = 0
+    while mvr:
+        mvr -= 1
+        if str_[mvr] == "]":
+            level += 1
+        elif str_[mvr] == "[":
+            if level:
+                level -= 1
+            else:
+                break
+    if not (mvr >= 0 and str_[mvr] == "["):
+        return
+    first_part = str_[mvr : index_ + 1]
+    mvr = index_ + 1
+    level = 0
+    while mvr < len(str_):
+        mvr += 1
+        if str_[mvr] == "(":
+            level += 1
+        elif str_[mvr] == ")":
+            if level:
+                level -= 1
+            else:
+                break
+    if not (mvr < len(str_) and str_[mvr] == ")"):
+        return
+    second_part = str_[index_ + 1 : mvr + 1]
+    if first_part[1:5] == "http" and second_part[1:5] == "http":
+        link = first_part[1:-1]
+    else:
+        link = None
+    return (first_part, second_part, link)
 
 
 def fix_trello_new_editor_links(desc):
     srch = RE_LINK.search(desc)
+    result = []
     while srch:
-        actual_link = srch.group(2).split()[0]
-        desc = desc.replace(srch.group(0), actual_link)
+        span = srch.span()
+        link_parsed = find_and_parse_link(desc, span[0])
+        if link_parsed and link_parsed[2]:
+            together = link_parsed[0] + link_parsed[1]
+            end = desc.find(together) + len(together) + 1
+            result.append(desc[:end].replace(together, link_parsed[2]))
+            desc = desc[end:]
+        else:
+            result.append(desc[: span[1]])
+            desc = desc[span[1] :]
         srch = RE_LINK.search(desc)
-    return desc
+    if not result:
+        return desc
+    else:
+        result.append(desc)
+    return "".join(result)
 
 
 def process_desc(s, onlyanswers=False, noanswers=False):
@@ -211,10 +263,14 @@ def gui_trello_download(args, sourcedir):
 
     json_ = json.loads(req.content.decode("utf8"))
     _names = defaultdict(lambda: None)
-    open_lists = list(filter(lambda l: not l["closed"], json_["lists"]))
+    open_lists = list(filter(lambda x: not x["closed"], json_["lists"]))
     for list_ in open_lists:
         _names[list_["id"]] = list_["name"].replace("/", "_")
         _list_counters[list_["id"]] = 0
+    if args.lists:
+        good_lists = [x.strip() for x in args.lists.split(",")]
+    else:
+        good_lists = None
 
     if args.si:
         _docs = defaultdict(lambda: Document(template_path))
@@ -223,13 +279,21 @@ def gui_trello_download(args, sourcedir):
         qb_doc = Document(template_path)
 
     for card in json_["cards"]:
-        if args.replace_double_line_breaks or args.fix_trello_new_editor:
-            card["desc"] = card["desc"].replace("\n\n", "\n")
-        if args.fix_trello_new_editor:
+        if args.replace_double_line_breaks or args.fix_trello_new_editor == "on":
+            card["desc"] = card["desc"].replace("\n\n", "\n").replace("\\@", "@")
+            card["desc"] = re.sub("\n +", "\n", card["desc"])
+            card["desc"] = card["desc"].replace("\n\\-", "\n-")
+            card["desc"] = card["desc"].replace("\\#", "#")
+            card["desc"] = card["desc"].replace("```", "")
+        if args.fix_trello_new_editor == "on":
             card["desc"] = fix_trello_new_editor_links(card["desc"])
         list_id = card["idList"]
         list_name = _names[list_id]
-        if card.get("closed") or list_name is None:
+        if (
+            card.get("closed")
+            or list_name is None
+            or (good_lists and list_name not in good_lists)
+        ):
             continue
 
         _list_counters[list_id] += 1
@@ -362,9 +426,9 @@ def get_token(tokenpath):
     return token
 
 
-def gui_trello(args, sourcedir=None):
+def gui_trello(args):
     csdir = get_chgksuite_dir()
-    resourcedir = os.path.join(sourcedir, "resources")
+    sourcedir, resourcedir = get_source_dirs()
     tokenpath = os.path.join(csdir, ".trello_token")
     if not os.path.isfile(tokenpath):
         token = get_token(tokenpath)
@@ -382,12 +446,3 @@ def gui_trello(args, sourcedir=None):
         gui_trello_upload(args, sourcedir)
     elif args.trellosubcommand == "token":
         get_token(tokenpath)
-
-
-def main():
-    print("This program was not designed to run standalone.")
-    input("Press Enter to continue...")
-
-
-if __name__ == "__main__":
-    main()
